@@ -1,5 +1,5 @@
 # #
-# Copyright 2009-2013 Ghent University
+# Copyright 2009-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -33,6 +33,7 @@ The EasyBlock class should serve as a base class for all easyblocks.
 @author: Jens Timmerman (Ghent University)
 @author: Toon Willems (Ghent University)
 @author: Ward Poelmans (Ghent University)
+@author: Fotis Georgatos (University of Luxembourg)
 """
 
 import copy
@@ -41,28 +42,36 @@ import re
 import os
 import shutil
 import stat
+import sys
 import time
+import traceback
 import urllib
 from distutils.version import LooseVersion
 from vsc import fancylogger
 from vsc.utils.missing import nub
 
 import easybuild.tools.environment as env
+from easybuild.tools import config, filetools
 from easybuild.framework.easyconfig.default import get_easyconfig_parameter_default
 from easybuild.framework.easyconfig.easyconfig import EasyConfig, ITERATE_OPTIONS, resolve_template
-from easybuild.framework.easyconfig.tools import get_paths_for
+from easybuild.framework.easyconfig.tools import get_paths_for, resolve_dependencies
 from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_EASYBLOCK_RUN_STEP
-from easybuild.tools.build_log import EasyBuildError, print_msg
-from easybuild.tools.config import build_path, install_path, log_path, get_log_filename
-from easybuild.tools.config import read_only_installdir, source_paths, module_classes
+from easybuild.tools.build_details import get_build_stats
+from easybuild.tools.build_log import EasyBuildError, print_error, print_msg
+from easybuild.tools.config import build_path, get_log_filename, get_repository, get_repositorypath, install_path
+from easybuild.tools.config import log_path, module_classes, read_only_installdir, source_paths
 from easybuild.tools.environment import modify_env
-from easybuild.tools.filetools import adjust_permissions, apply_patch, convert_name, download_file
-from easybuild.tools.filetools import encode_class_name, extract_file, run_cmd, rmtree2
+from easybuild.tools.filetools import DEFAULT_CHECKSUM
+from easybuild.tools.filetools import adjust_permissions, apply_patch, convert_name
+from easybuild.tools.filetools import download_file, encode_class_name, extract_file, read_file, rmtree2, run_cmd
 from easybuild.tools.filetools import decode_class_name, write_file, compute_checksum, verify_checksum
+from easybuild.tools.jenkins import write_to_xml
 from easybuild.tools.module_generator import GENERAL_CLASS, ModuleGenerator
 from easybuild.tools.module_generator import det_full_module_name, det_devel_module_filename
+from easybuild.tools.module_naming_scheme.utilities import det_full_ec_version
 from easybuild.tools.modules import ROOT_ENV_VAR_NAME_PREFIX, VERSION_ENV_VAR_NAME_PREFIX, DEVEL_ENV_VAR_NAME_PREFIX
 from easybuild.tools.modules import get_software_root, modules_tool
+from easybuild.tools.repository import init_repository
 from easybuild.tools.toolchain import DUMMY_TOOLCHAIN_NAME
 from easybuild.tools.systemtools import get_avail_core_count
 from easybuild.tools.utilities import remove_unwanted_chars
@@ -91,10 +100,15 @@ class EasyBlock(object):
     #
     # INIT
     #
-    def __init__(self, path, debug=False, robot_path=None, validate_ec=True, silent=False, check_osdeps=True):
+    def __init__(self, path, build_options=None, build_specs=None):
         """
         Initialize the EasyBlock instance.
+        @param path: path to easyconfig file
+        @param build_options: dictionary of build options, e.g. robot_path, validate_ec, check_osdeps, ... (default: {})
+        @param build_specs: dictionary of build specifications (see EasyConfig class, default: {})
         """
+        if build_options is None:
+            build_options = {}
 
         # list of patch/source files, along with checksums
         self.patches = []
@@ -117,16 +131,25 @@ class EasyBlock(object):
         # module generator
         self.moduleGenerator = None
 
+        # modules footer
+        self.modules_footer = None
+        modules_footer_path = build_options.get('modules_footer', None)
+        if modules_footer_path is not None:
+            self.modules_footer = read_file(modules_footer_path)
+
+        # recursive unloading in modules
+        self.recursive_mod_unload = build_options.get('recursive_mod_unload', False)
+
         # easyconfig for this application
         all_stops = [x[0] for x in self.get_steps()]
-        self.cfg = EasyConfig(
-            path,
-            extra_options=self.extra_options(),
-            validate=validate_ec,
-            valid_module_classes=module_classes(),
-            valid_stops=all_stops,
-            check_osdeps=check_osdeps,
-        )
+        ec_build_options = copy.deepcopy(build_options)
+        ec_build_options.update({
+            'valid_module_classes': module_classes(),
+            'valid_stops': all_stops,
+            'validate': build_options.get('validate_ec', True),
+        })
+        extra = self.extra_options()
+        self.cfg = EasyConfig(path, extra_options=extra, build_options=ec_build_options, build_specs=build_specs)
 
         # indicates whether build should be performed in installation dir
         self.build_in_installdir = self.cfg['buildininstalldir']
@@ -134,7 +157,7 @@ class EasyBlock(object):
         # logging
         self.log = None
         self.logfile = None
-        self.logdebug = debug
+        self.logdebug = build_options.get('debug', False)
         self.postmsg = ''  # allow a post message to be set, which can be shown as last output
 
         # original environ will be set later
@@ -144,7 +167,7 @@ class EasyBlock(object):
         self.loaded_modules = []
 
         # robot path
-        self.robot_path = robot_path
+        self.robot_path = build_options.get('robot_path', None)
 
         # original module path
         self.orig_modulepath = os.getenv('MODULEPATH')
@@ -162,7 +185,7 @@ class EasyBlock(object):
         self.sanity_check_fail_msgs = []
 
         # should we keep quiet?
-        self.silent = silent
+        self.silent = build_options.get('silent', False)
 
         # full module name to generate
         self.mod_name = None
@@ -343,6 +366,8 @@ class EasyBlock(object):
                                'options': ext_options,
                               }
 
+                    checksums = ext_options.get('checksums', None)
+
                     if ext_options.get('source_tmpl', None):
                         fn = resolve_template(ext_options['source_tmpl'], ext_src)
                     else:
@@ -357,10 +382,26 @@ class EasyBlock(object):
                         if src_fn:
                             ext_src.update({'src': src_fn})
 
+                            if checksums:
+                                fn_checksum = self.get_checksum_for(checksums, filename=src_fn, index=0)
+                                if verify_checksum(src_fn, fn_checksum):
+                                    self.log.info('Checksum for ext source %s verified' % fn)
+                                else:
+                                    self.log.error('Checksum for ext source %s failed' % fn)
+
                             ext_patches = self.fetch_patches(ext_options.get('patches', []), extension=True)
                             if ext_patches:
                                 self.log.debug('Found patches for extension %s: %s' % (ext_name, ext_patches))
                                 ext_src.update({'patches': ext_patches})
+
+                                if checksums:
+                                    self.log.debug('Verifying checksums for extension patches...')
+                                    for index, ext_patch in enumerate(ext_patches):
+                                        checksum = self.get_checksum_for(checksums[1:], filename=ext_patch, index=index)
+                                        if verify_checksum(ext_patch, checksum):
+                                            self.log.info('Checksum for extension patch %s verified' % ext_patch)
+                                        else:
+                                            self.log.error('Checksum for extension patch %s failed' % ext_patch)
                             else:
                                 self.log.debug('No patches found for extension %s.' % ext_name)
 
@@ -573,14 +614,16 @@ class EasyBlock(object):
 
         builddir = os.path.join(os.path.abspath(build_path()), clean_name, self.version, lastdir)
 
-        # make sure build dir is unique
-        uniq_builddir = builddir
-        suff = 0
-        while(os.path.isdir(uniq_builddir)):
-            uniq_builddir = "%s.%d" % (builddir, suff)
-            suff += 1
+        # make sure build dir is unique if cleanupoldbuild is False or not set
+        if not self.cfg.get('cleanupoldbuild', False):
+            uniq_builddir = builddir
+            suff = 0
+            while(os.path.isdir(uniq_builddir)):
+                uniq_builddir = "%s.%d" % (builddir, suff)
+                suff += 1
+            builddir = uniq_builddir
 
-        self.builddir = uniq_builddir
+        self.builddir = builddir
         self.log.info("Build dir set to %s" % self.builddir)
 
     def make_builddir(self):
@@ -707,7 +750,7 @@ class EasyBlock(object):
                     path = os.environ[key]
                     if os.path.isfile(path):
                         mod_name = path.rsplit(os.path.sep, 1)[-1]
-                        load_txt += mod_gen.load_module(mod_name)
+                        load_txt += mod_gen.load_module(mod_name, recursive_unload=self.recursive_mod_unload)
 
         if create_in_builddir:
             output_dir = self.builddir
@@ -719,7 +762,7 @@ class EasyBlock(object):
         filename = os.path.join(output_dir, det_devel_module_filename(self.cfg))
         self.log.debug("Writing devel module to %s" % filename)
 
-        write_file(filename, header+load_txt+env_txt)
+        write_file(filename, header + load_txt + env_txt)
 
         # cleanup: unload fake module, remove fake module dir
         self.clean_up_fake_module(fake_mod_data)
@@ -732,8 +775,9 @@ class EasyBlock(object):
 
         # Load toolchain
         if self.toolchain.name != DUMMY_TOOLCHAIN_NAME:
-            load += self.moduleGenerator.load_module(self.toolchain.det_module_name())
-            unload += self.moduleGenerator.unload_module(self.toolchain.det_module_name())
+            tc_mod_name = self.toolchain.det_module_name()
+            load += self.moduleGenerator.load_module(tc_mod_name, recursive_unload=self.recursive_mod_unload)
+            unload += self.moduleGenerator.unload_module(tc_mod_name)
 
         # Load dependencies
         builddeps = self.cfg.builddependencies()
@@ -741,7 +785,7 @@ class EasyBlock(object):
             if not dep in builddeps:
                 dep_mod_name = det_full_module_name(dep)
                 self.log.debug("Adding %s as a module dependency" % dep_mod_name)
-                load += self.moduleGenerator.load_module(dep_mod_name)
+                load += self.moduleGenerator.load_module(dep_mod_name, recursive_unload=self.recursive_mod_unload)
                 unload += self.moduleGenerator.unload_module(dep_mod_name)
             else:
                 self.log.debug("Skipping build dependency %s" % str(dep))
@@ -789,6 +833,7 @@ class EasyBlock(object):
         """
         Sets optional variables for extensions.
         """
+        # add stuff specific to individual extensions
         txt = self.module_extra_extensions
 
         # set environment variable that specifies list of extensions
@@ -798,6 +843,23 @@ class EasyBlock(object):
 
         return txt
 
+    def make_module_footer(self):
+        """
+        Insert a footer section in the modulefile, primarily meant for contextual information
+        """
+        txt = '\n# Built with EasyBuild version %s\n' % VERBOSE_VERSION
+
+        # add extra stuff for extensions (if any)
+        if self.cfg['exts_list']:
+            txt += self.make_module_extra_extensions()
+
+        # include modules footer if one is specified
+        if self.modules_footer is not None:
+            self.log.debug("Including specified footer into module: '%s'" % self.modules_footer)
+            txt += self.modules_footer
+
+        return txt
+
     def make_module_req(self):
         """
         Generate the environment-variables to run the module.
@@ -818,6 +880,7 @@ class EasyBlock(object):
         return {
             'PATH': ['bin', 'sbin'],
             'LD_LIBRARY_PATH': ['lib', 'lib64'],
+            'LIBRARY_PATH': ['lib', 'lib64'],
             'CPATH':['include'],
             'MANPATH': ['man', 'share/man'],
             'PKG_CONFIG_PATH' : ['lib/pkgconfig', 'share/pkgconfig'],
@@ -1198,12 +1261,12 @@ class EasyBlock(object):
         else:
             self.log.info('no patches provided')
 
-        # compute md5 checksums for all source and patch files
+        # compute checksums for all source and patch files
         if not skip_checksums:
             for fil in self.src + self.patches:
-                md5_sum = compute_checksum(fil['path'], checksum_type='md5')
-                fil['md5'] = md5_sum
-                self.log.info("MD5 checksum for %s: %s" % (fil['path'], fil['md5']))
+                check_sum = compute_checksum(fil['path'], checksum_type=DEFAULT_CHECKSUM)
+                fil[DEFAULT_CHECKSUM] = check_sum
+                self.log.info("%s checksum for %s: %s" % (DEFAULT_CHECKSUM, fil['path'], fil[DEFAULT_CHECKSUM]))
 
         # set level of parallelism for build
         self.set_parallelism()
@@ -1638,9 +1701,7 @@ class EasyBlock(object):
         txt += self.make_module_dep()
         txt += self.make_module_req()
         txt += self.make_module_extra()
-        if self.cfg['exts_list']:
-            txt += self.make_module_extra_extensions()
-        txt += '\n# built with EasyBuild version %s\n' % VERBOSE_VERSION
+        txt += self.make_module_footer()
 
         write_file(self.moduleGenerator.filename, txt)
 
@@ -1823,7 +1884,6 @@ class EasyBlock(object):
         return True
 
 
-
 def get_class_for(modulepath, class_name):
     """
     Get class for a given class name and easyblock module path.
@@ -1840,6 +1900,7 @@ def get_class_for(modulepath, class_name):
         raise ImportError
     return c
 
+
 def get_module_path(name, generic=False, decode=True):
     """
     Determine the module path for a given easyblock or software name,
@@ -1860,6 +1921,7 @@ def get_module_path(name, generic=False, decode=True):
 
     return '.'.join([modpath, module_name])
 
+
 def get_class(easyblock, name=None):
     """
     Get class for a particular easyblock (or use default)
@@ -1927,6 +1989,294 @@ def get_class(easyblock, name=None):
     except Exception, err:
         _log.error("Failed to obtain class for %s easyblock (not available?): %s" % (easyblock, err))
 
+
+def build_and_install_software(module, orig_environ, build_options=None, build_specs=None):
+    """
+    Build the software
+    @param module: dictionary contaning parsed easyconfig + metadata
+    @param orig_environ: original environment (used to reset environment)
+    @param build_options: dictionary specifying build options (e.g. robot_path, check_osdeps, ...)
+    @param build_specs: dictionary specifying build specifications (e.g. version, toolchain, ...)
+    """
+    silent = build_options.get('silent', False)
+
+    spec = module['spec']
+
+    print_msg("processing EasyBuild easyconfig %s" % spec, log=_log, silent=silent)
+
+    # restore original environment
+    _log.info("Resetting environment")
+    filetools.errors_found_in_log = 0
+    modify_env(os.environ, orig_environ)
+
+    cwd = os.getcwd()
+
+    # load easyblock
+    easyblock = build_options.get('easyblock', None)
+    if not easyblock:
+        # try to look in .eb file
+        reg = re.compile(r"^\s*easyblock\s*=(.*)$")
+        txt = read_file(spec)
+        for line in txt.split('\n'):
+            match = reg.search(line)
+            if match:
+                easyblock = eval(match.group(1))
+                break
+
+    name = module['ec']['name']
+    try:
+        app_class = get_class(easyblock, name=name)
+        app = app_class(spec, build_options=build_options, build_specs=build_specs)
+        _log.info("Obtained application instance of for %s (easyblock: %s)" % (name, easyblock))
+    except EasyBuildError, err:
+        tup = (name, easyblock, err.msg)
+        print_error("Failed to get application instance for %s (easyblock: %s): %s" % tup, silent=silent)
+
+    # application settings
+    stop = build_options.get('stop', None)
+    if stop is not None:
+        _log.debug("Stop set to %s" % stop)
+        app.cfg['stop'] = stop
+
+    skip = build_options.get('skip', None)
+    if skip is not None:
+        _log.debug("Skip set to %s" % skip)
+        app.cfg['skip'] = skip
+
+    # build easyconfig
+    errormsg = '(no error)'
+    # timing info
+    start_time = time.time()
+    try:
+        run_test_cases = not build_options.get('skip_test_cases', False) and app.cfg['tests']
+        regtest_online = build_options.get('regtest_online', False)
+        result = app.run_all_steps(run_test_cases=run_test_cases, regtest_online=regtest_online)
+    except EasyBuildError, err:
+        lastn = 300
+        errormsg = "autoBuild Failed (last %d chars): %s" % (lastn, err.msg[-lastn:])
+        _log.exception(errormsg)
+        result = False
+
+    ended = "ended"
+
+    # successful build
+    if result:
+
+        if app.cfg['stop']:
+            ended = "STOPPED"
+            if app.builddir is not None:
+                new_log_dir = os.path.join(app.builddir, config.log_path())
+            else:
+                new_log_dir = os.path.dirname(app.logfile)
+        else:
+            new_log_dir = os.path.join(app.installdir, config.log_path())
+
+            # collect build stats
+            _log.info("Collecting build stats...")
+
+            buildstats = get_build_stats(app, start_time, build_options.get('command_line', None))
+            _log.info("Build stats: %s" % buildstats)
+
+            try:
+                # upload spec to central repository
+                currentbuildstats = app.cfg['buildstats']
+                repo = init_repository(get_repository(), get_repositorypath())
+                if 'originalSpec' in module:
+                    block = det_full_ec_version(app.cfg) + ".block"
+                    repo.add_easyconfig(module['originalSpec'], app.name, block, buildstats, currentbuildstats)
+                repo.add_easyconfig(spec, app.name, det_full_ec_version(app.cfg), buildstats, currentbuildstats)
+                repo.commit("Built %s" % det_full_module_name(app.cfg))
+                del repo
+            except EasyBuildError, err:
+                _log.warn("Unable to commit easyconfig to repository: %s", err)
+
+        exit_code = 0
+        succ = "successfully"
+        summary = "COMPLETED"
+
+        # cleanup logs
+        app.close_log()
+        try:
+            if not os.path.isdir(new_log_dir):
+                os.makedirs(new_log_dir)
+            log_fn = os.path.basename(get_log_filename(app.name, app.version))
+            application_log = os.path.join(new_log_dir, log_fn)
+            shutil.move(app.logfile, application_log)
+            _log.debug("Moved log file %s to %s" % (app.logfile, application_log))
+        except (IOError, OSError), err:
+            print_error("Failed to move log file %s to new log file %s: %s" % (app.logfile, application_log, err))
+
+        try:
+            newspec = os.path.join(new_log_dir, "%s-%s.eb" % (app.name, det_full_ec_version(app.cfg)))
+            shutil.copy(spec, newspec)
+            _log.debug("Copied easyconfig file %s to %s" % (spec, newspec))
+        except (IOError, OSError), err:
+            print_error("Failed to move easyconfig %s to log dir %s: %s" % (spec, new_log_dir, err))
+
+    # build failed
+    else:
+        exit_code = 1
+        summary = "FAILED"
+
+        build_dir = ''
+        if app.builddir:
+            build_dir = " (build directory: %s)" % (app.builddir)
+        succ = "unsuccessfully%s:\n%s" % (build_dir, errormsg)
+
+        # cleanup logs
+        app.close_log()
+        application_log = app.logfile
+
+    print_msg("%s: Installation %s %s" % (summary, ended, succ), log=_log, silent=silent)
+
+    # check for errors
+    if exit_code != 0 or filetools.errors_found_in_log > 0:
+        print_msg("\nWARNING: Build exited with non-zero exit code %d. %d possible error(s) were detected in the "
+                  "build logs, please verify the build.\n" % (exit_code, filetools.errors_found_in_log),
+                  _log, silent=silent)
+
+    if app.postmsg:
+        print_msg("\nWARNING: %s\n" % app.postmsg, _log, silent=silent)
+
+    print_msg("Results of the build can be found in the log file %s" % application_log, _log, silent=silent)
+
+    del app
+    os.chdir(cwd)
+
+    return (exit_code == 0, application_log)
+
+
+def get_easyblock_instance(easyconfig, build_options=None, build_specs=None):
+    """
+    Get an instance for this easyconfig
+    @param easyconfig: parsed easyconfig
+    @param build_options: dictionary specifying build options (e.g. robot_path, check_osdeps, ...)
+    @param build_specs: dictionary specifying build specifications (e.g. version, toolchain, ...)
+
+    returns an instance of EasyBlock (or subclass thereof)
+    """
+    spec = easyconfig['spec']
+    name = easyconfig['ec']['name']
+
+    # handle easyconfigs with custom easyblocks
+    easyblock = None
+    reg = re.compile(r"^\s*easyblock\s*=(.*)$")
+    txt = read_file(spec)
+    for line in txt.split('\n'):
+        match = reg.search(line)
+        if match:
+            easyblock = eval(match.group(1))
+            break
+
+    app_class = get_class(easyblock, name=name)
+    return app_class(spec, build_options=build_options, build_specs=build_specs)
+
+
+def build_easyconfigs(easyconfigs, output_dir, test_results, build_options=None):
+    """Build the list of easyconfigs."""
+
+    build_stopped = {}
+    apploginfo = lambda x, y: x.log.info(y)
+
+    def perform_step(step, obj, method, logfile):
+        """Perform method on object if it can be built."""
+        if (isinstance(obj, dict) and obj['spec'] not in build_stopped) or obj not in build_stopped:
+
+            # update templates before every step (except for initialization)
+            if isinstance(obj, EasyBlock):
+                obj.update_config_template_run_step()
+
+            try:
+                if step == 'initialization':
+                    _log.info("Running %s step" % step)
+                    return get_easyblock_instance(obj, build_options=build_options)
+                else:
+                    apploginfo(obj, "Running %s step" % step)
+                    method(obj)
+            except Exception, err:  # catch all possible errors, also crashes in EasyBuild code itself
+                fullerr = str(err)
+                if not isinstance(err, EasyBuildError):
+                    tb = traceback.format_exc()
+                    fullerr = '\n'.join([tb, str(err)])
+                # we cannot continue building it
+                if step == 'initialization':
+                    obj = obj['spec']
+                test_results.append((obj, step, fullerr, logfile))
+                # keep a dict of so we can check in O(1) if objects can still be build
+                build_stopped[obj] = step
+
+    # initialize all instances
+    apps = []
+    for ec in easyconfigs:
+        instance = perform_step('initialization', ec, None, _log)
+        instance.mod_name = det_full_module_name(instance.cfg)
+        apps.append(instance)
+
+    base_dir = os.getcwd()
+    base_env = copy.deepcopy(os.environ)
+    succes = []
+
+    for app in apps:
+
+        # if initialisation step failed, app will be None
+        if app:
+
+            applog = os.path.join(output_dir, "%s-%s.log" % (app.name, det_full_ec_version(app.cfg)))
+
+            start_time = time.time()
+
+            # start with a clean slate
+            os.chdir(base_dir)
+            modify_env(os.environ, base_env)
+
+            steps = EasyBlock.get_steps(iteration_count=app.det_iter_cnt())
+
+            for (step_name, _, step_methods, skippable) in steps:
+                if skippable and step_name in app.cfg['skipsteps']:
+                    _log.info("Skipping step %s" % step_name)
+                else:
+                    for step_method in step_methods:
+                        method_name = '_'.join(step_method.func_code.co_names)
+                        perform_step('_'.join([step_name, method_name]), app, step_method, applog)
+
+            # close log and move it
+            app.close_log()
+            try:
+                # retain old logs
+                if os.path.exists(applog):
+                    i = 0
+                    old_applog = "%s.%d" % (applog, i)
+                    while os.path.exists(old_applog):
+                        i += 1
+                        old_applog = "%s.%d" % (applog, i)
+                    shutil.move(applog, old_applog)
+                    _log.info("Moved existing log file %s to %s" % (applog, old_applog))
+
+                shutil.move(app.logfile, applog)
+                _log.info("Log file moved to %s" % applog)
+            except IOError, err:
+                print_error("Failed to move log file %s to new log file %s: %s" % (app.logfile, applog, err))
+
+            if app not in build_stopped:
+                # gather build stats
+                buildstats = get_build_stats(app, start_time, build_options.get('command_line', None))
+                succes.append((app, buildstats))
+
+    for result in test_results:
+        _log.info("%s crashed with an error during fase: %s, error: %s, log file: %s" % result)
+
+    failed = len(build_stopped)
+    total = len(apps)
+
+    _log.info("%s of %s packages failed to build!" % (failed, total))
+
+    output_file = os.path.join(output_dir, "easybuild-test.xml")
+    _log.debug("writing xml output to %s" % output_file)
+    write_to_xml(succes, test_results, output_file)
+
+    return failed == 0
+
+
 class StopException(Exception):
     """
     StopException class definition.
