@@ -1,5 +1,5 @@
 # #
-# Copyright 2013-2013 Ghent University
+# Copyright 2013-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -27,14 +27,15 @@
 This describes the easyconfig version class. To be used in EasyBuild for anything related to version checking
 
 @author: Stijn De Weirdt (Ghent University)
+@author: Kenneth Hoste (Ghent University)
 """
-
+import copy
 import operator as op
 import re
 from distutils.version import LooseVersion
 from vsc import fancylogger
 
-from easybuild.tools.configobj import Section
+from easybuild.tools.convert import Convert, DictOfStrings, ListOfStrings
 from easybuild.tools.toolchain.utilities import search_toolchain
 
 
@@ -64,8 +65,14 @@ class VersionOperator(object):
     INCLUDE_OPERATORS = ['==', '>=', '<=']  # these operators *include* the (version) boundary
     ORDERED_OPERATORS = ['==', '>', '>=', '<', '<=']  # ordering by strictness
     OPERATOR_FAMILIES = [['>', '>='], ['<', '<=']]  # similar operators
+
+    # default version and operator when version is undefined
     DEFAULT_UNDEFINED_VERSION = EasyVersion('0.0.0')
-    DEFAULT_UNDEFINED_OPERATOR = OPERATOR_MAP['>']
+    DEFAULT_UNDEFINED_VERSION_OPERATOR = OPERATOR_MAP['>']
+    # default operator when operator is undefined (but version is)
+    DEFAULT_UNDEFINED_OPERATOR = OPERATOR_MAP['==']
+
+    DICT_SEPARATOR = ':'
 
     def __init__(self, versop_str=None, error_on_parse_failure=False):
         """
@@ -80,6 +87,7 @@ class VersionOperator(object):
         self.version_str = None
         self.version = None
         self.operator = None
+        self.suffix = None
         self.regex = self.versop_regex()
 
         self.error_on_parse_failure = error_on_parse_failure
@@ -102,18 +110,17 @@ class VersionOperator(object):
     __nonzero__ = __bool__
 
     def is_valid(self):
-        """Check if this is a valid VersionOperator"""
+        """Check if this is a valid VersionOperator. Suffix can be anything."""
         return not(self.version is None or self.operator is None)
 
     def set(self, versop_str):
         """
-        Parse argument and set attributes.
-        Returns None in case of failure (e.g. if supplied string doesn't parse), True in case of success.
+        Parse argument as a version operator, and set attributes.
+        Returns True in case of success, throws an error in case of parsing failure.
         """
         versop_dict = self.parse_versop_str(versop_str)
         if versop_dict is None:
-            self.log.warning("set('%s'): Failed to parse argument" % versop_str)
-            return None
+            self.log.error("Failed to parse '%s' as a version operator string" % versop_str)
         else:
             for k, v in versop_dict.items():
                 setattr(self, k, v)
@@ -122,6 +129,7 @@ class VersionOperator(object):
     def test(self, test_version):
         """
         Convert argument to an EasyVersion instance if needed, and return self.operator(<argument>, self.version)
+            Versions only, no suffix.
         @param test_version: a version string or EasyVersion instance
         """
         # checks whether this VersionOperator instance is valid using __bool__ function
@@ -142,11 +150,22 @@ class VersionOperator(object):
     def __str__(self):
         """Return string representation of this VersionOperator instance"""
         if self.operator is None:
-            operator = self.DEFAULT_UNDEFINED_OPERATOR
+            if self.version is None:
+                operator = self.DEFAULT_UNDEFINED_VERSION_OPERATOR
+            else:
+                operator = self.DEFAULT_UNDEFINED_OPERATOR
         else:
             operator = self.operator
         operator_str = self.REVERSE_OPERATOR_MAP[operator]
-        return ''.join(map(str, [operator_str, self.SEPARATOR, self.version]))
+
+        tmp = [operator_str, self.SEPARATOR, self.version]
+        if self.suffix is not None:
+            tmp.extend([self.SEPARATOR, self.suffix])
+        return ''.join(map(str, tmp))
+
+    def get_version_str(self):
+        """Return string representation of version (ignores operator)."""
+        return str(self.version)
 
     def __repr__(self):
         """Return instance as string (ignores begin_end)"""
@@ -154,9 +173,11 @@ class VersionOperator(object):
 
     def __eq__(self, versop):
         """Compare this instance to supplied argument."""
-        if not isinstance(versop, self.__class__):
+        if versop is None:
+            return False
+        elif not isinstance(versop, self.__class__):
             self.log.error("Types don't match in comparison: %s, expected %s" % (type(versop), self.__class__))
-        return self.version == versop.version and self.operator == versop.operator
+        return self.version == versop.version and self.operator == versop.operator and self.suffix == versop.suffix
 
     def __ne__(self, versop):
         """Is self not equal to versop"""
@@ -178,10 +199,19 @@ class VersionOperator(object):
         # - operator_str part is optional
         # - version_str should start/end with any word character except separator
         # - minimal version_str length is 1
-        reg_text = r"(?:(?P<operator_str>%(ops)s)%(sep)s)?(?P<version_str>[^%(sep)s\W](?:\S*[^%(sep)s\W])?)" % {
+        # - optional extensions:
+        #    - suffix: the version suffix
+        reg_text_operator = r"(?:(?P<operator_str>%(ops)s)%(sep)s)?" % {
             'sep': self.SEPARATOR,
             'ops': '|'.join(operators),
         }
+        reg_text_version = r"(?P<version_str>[^%(sep)s\W](?:\S*[^%(sep)s\W])?)" % { 'sep': self.SEPARATOR }
+        reg_text_ext = r"(?:%(sep)s(?:suffix%(extsep)s(?P<suffix>[^%(sep)s]+)))?" % {
+            'sep': self.SEPARATOR,
+            'extsep': self.DICT_SEPARATOR,
+        }
+
+        reg_text = r"%s%s%s" % (reg_text_operator, reg_text_version, reg_text_ext)
         if begin_end:
             reg_text = r"^%s$" % reg_text
         reg = re.compile(reg_text)
@@ -204,12 +234,15 @@ class VersionOperator(object):
         self.log.debug('converted string %s to version %s' % (version_str, version))
         return version
 
-    def _convert_operator(self, operator_str):
+    def _convert_operator(self, operator_str, version=None):
         """Return the operator"""
         operator = None
         if operator_str is None:
-            operator = self.DEFAULT_UNDEFINED_OPERATOR
-            self.log.warning('_convert: operator_str None, set it to DEFAULT_UNDEFINED_OPERATOR %s' % operator)
+            if version == self.DEFAULT_UNDEFINED_VERSION or version is None:
+                operator = self.DEFAULT_UNDEFINED_VERSION_OPERATOR
+            else:
+                operator = self.DEFAULT_UNDEFINED_OPERATOR
+            self.log.warning('_convert: operator_str None, set it to default operator (with version: %s) %s' % (operator, version))
         elif operator_str in self.OPERATOR_MAP:
             operator = self.OPERATOR_MAP[operator_str]
         else:
@@ -237,8 +270,11 @@ class VersionOperator(object):
         if not 'versop_str' in versop_dict:
             self.log.error('Missing versop_str in versop_dict %s' % versop_dict)
 
-        versop_dict['version'] = self._convert(versop_dict['version_str'])
-        versop_dict['operator'] = self._convert_operator(versop_dict['operator_str'])
+        version = self._convert(versop_dict['version_str'])
+        operator = self._convert_operator(versop_dict['operator_str'], version=version)
+
+        versop_dict['version'] = version
+        versop_dict['operator'] = operator
         self.log.debug('versop expression %s parsed into versop_dict %s' % (versop_dict['versop_str'], versop_dict))
 
         return versop_dict
@@ -258,6 +294,9 @@ class VersionOperator(object):
             '> 3' and '== 3' : no overlap
             '>= 3' and '== 3' : overlap, and conflict (boundary 3 is ambigous)
             '> 3' and '>= 3' : overlap, no conflict ('> 3' is more strict then '>= 3')
+            
+            # suffix
+            '> 2 suffix:-x1' > '> 1 suffix:-x2': suffix not equal, conflict
         """
         versop_msg = "this versop %s and versop_other %s" % (self, versop_other)
 
@@ -270,6 +309,8 @@ class VersionOperator(object):
         boundary_self_in_other = versop_other.test(self.version)
         boundary_other_in_self = self.test(versop_other.version)
 
+        suffix_allowed = self.suffix == versop_other.suffix
+
         same_family = False
         for fam in self.OPERATOR_FAMILIES:
             fam_op = [self.OPERATOR_MAP[x] for x in fam]
@@ -285,15 +326,15 @@ class VersionOperator(object):
             if same_boundary:
                 if op.xor(self_includes_boundary, other_includes_boundary):
                     self.log.debug("%s, one includes boundary and one is strict => overlap, no conflict" % msg)
-                    return True, False
+                    overlap_conflict = (True, False)
                 else:
                     # conflict
                     self.log.debug("%s, and both include the boundary => overlap and conflict" % msg)
-                    return True, True
+                    overlap_conflict = (True, True)
             else:
                 # conflict
                 self.log.debug("%s, and different boundaries => overlap and conflict" % msg)
-                return True, True
+                overlap_conflict = (True, True)
         else:
             # both boundaries not included in one other version expression
             # => never a conflict, only possible overlap
@@ -309,7 +350,14 @@ class VersionOperator(object):
                 # overlap if boundary of one is in other
                 overlap = boundary_self_in_other or boundary_other_in_self
             self.log.debug("No conflict between %s; %s overlap %s, no conflict" % (versop_msg, msg, overlap))
-            return overlap, False
+            overlap_conflict = (overlap, False)
+
+        if not suffix_allowed:
+            # always conflict
+            self.log.debug("Suffix for %s are not equal. Force conflict True." % versop_msg)
+            overlap_conflict = (overlap_conflict[0], True)
+
+        return overlap_conflict
 
     def __gt__(self, versop_other):
         """
@@ -326,6 +374,10 @@ class VersionOperator(object):
             '== 4' > '> 3' : equality is more strict than inequality, but this order by boundaries
             '> 3' > '== 2' : there is no overlap, so just order the intervals according their boundaries
             '> 1' > '== 1' > '< 1' : no overlap, same boundaries, order by operator
+            suffix:
+                '> 2' > '> 1': both equal (both None), ordering like above
+                '> 2 suffix:-x1' > '> 1 suffix:-x1': both equal (both -x1), ordering like above
+                '> 2 suffix:-x1' > '> 1 suffix:-x2': not equal, conflict
         """
         overlap, conflict = self.test_overlap_and_conflict(versop_other)
         versop_msg = "this versop %s and versop_other %s" % (self, versop_other)
@@ -355,7 +407,9 @@ class VersionOperator(object):
         return is_gt
 
     def _gt_safe(self, version_gt_op, versop_other):
-        """Conflict free comparsion by version first, and if versions are equal, by operator"""
+        """Conflict free comparsion by version first, and if versions are equal, by operator. 
+            Suffix are not considered.
+        """
         if len(self.ORDERED_OPERATORS) != len(self.OPERATOR_MAP):
             self.log.error('Inconsistency between ORDERED_OPERATORS and OPERATORS (lists are not of same length)')
 
@@ -404,6 +458,20 @@ class ToolchainVersionOperator(VersionOperator):
         known_tc_name = self.tc_name in tc_names
         return known_tc_name and super(ToolchainVersionOperator, self).is_valid()
 
+    def set(self, tcversop_str):
+        """
+        Parse argument as toolchain version string, and set attributes.
+        Returns None in case of failure (e.g. if supplied string doesn't parse), True in case of success.
+        """
+        versop_dict = self.parse_versop_str(tcversop_str)
+        if versop_dict is None:
+            self.log.warning("Failed to parse '%s' as a toolchain version operator string" % tcversop_str)
+            return None
+        else:
+            for k, v in versop_dict.items():
+                setattr(self, k, v)
+            return True
+
     def versop_regex(self):
         """
         Create the regular expression for toolchain support of format ^<toolchain> <versop_expr>$ ,
@@ -436,9 +504,29 @@ class ToolchainVersionOperator(VersionOperator):
 
         tcversop_dict = super(ToolchainVersionOperator, self).parse_versop_str(None, versop_dict=tcversop_dict)
 
+        if tcversop_dict.get('version_str', None) is not None and tcversop_dict.get('operator_str', None) is None:
+            self.log.error("Toolchain version found, but no operator (use ' == '?).")
+
         self.log.debug("toolchain versop expression '%s' parsed to '%s'" % (tcversop_str, tcversop_dict))
         return tcversop_dict
 
+    def as_dict(self):
+        """
+        Return toolchain version operator as a dictionary with name/version keys.
+        Returns None if translation to a dictionary is not possible (e.g. non-equals operator, missing version, ...).
+        """
+        version = self.get_version_str()
+        if self.operator == self.OPERATOR_MAP['==']:
+            tc_dict = {
+                'name': self.tc_name,
+                'version': version,
+            }
+            if self.suffix is not None:
+                tc_dict.update({'versionsuffix': self.suffix})
+            return tc_dict
+        else:
+            return None
+
 
 class OrderedVersionOperators(object):
     """
@@ -458,8 +546,8 @@ class OrderedVersionOperators(object):
         self.datamap = {}
 
     def __str__(self):
-        """Print the list"""
-        return str(self.versops)
+        """Print the list and map"""
+        return "ordered version operators: %s; data map: %s" % (self.versops, self.datamap)
 
     def add(self, versop_new, data=None):
         """
@@ -499,225 +587,3 @@ class OrderedVersionOperators(object):
 
                 self.log.debug("Keeping track of data for %s: %s" % (versop_new, data))
                 self.datamap[versop_new] = data
-
-class ConfigObjVersion(object):
-    """
-    ConfigObj version checker
-    - first level sections except default
-      - check toolchain
-      - check version
-    - second level
-      - version : dependencies
-
-    Given ConfigObj instance, make instance that can check if toolchain/version is allowed,
-    return version, toolchain name, toolchain version and dependency
-
-    Mandatory/minimal (to mimic v1.0 behaviour)
-    [DEFAULT]
-    versions=version_operator
-    toolchains=toolchain_version_operator
-
-    Optional
-    [DEFAULT]
-    [[SUPPORTED]]
-    toolchains=toolchain_versop[,...]
-    versions=versop[,...]
-    [<operatorX> <versionX>]
-    [<operatorY> <versionY>]
-    [<toolchainA> <operatorA> <versionA>]
-    [<toolchainB> <operatorB> <versionB>]
-    
-    """
-    # TODO: add nested/recursive example to docstring
-
-    DEFAULT = 'DEFAULT'
-    # list of known marker types (except default)
-    KNOWN_MARKER_TYPES = [ToolchainVersionOperator, VersionOperator]  # order matters, see parse_sections
-    VERSION_OPERATOR_VALUE_TYPES = {
-        'toolchains': ToolchainVersionOperator,
-        'versions': VersionOperator,
-    }
-
-    def __init__(self, configobj=None):
-        """
-        Initialise ConfigObjVersion instance
-        @param configobj: ConfigObj instance
-        """
-        self.log = fancylogger.getLogger(self.__class__.__name__, fname=False)
-
-        self.tcname = None
-
-        self.default = {}  # default section
-        self.sections = {}  # non-default sections
-        self.unfiltered_sections = {}  # unfiltered non-default sections
-
-        self.versops = OrderedVersionOperators()
-        self.tcversops = OrderedVersionOperators()
-
-        if configobj is not None:
-            self.parse(configobj)
-
-    def parse_sections(self, configobj, toparse=None, parent=None, depth=0):
-        """
-        Parse configobj instance; convert all supported sections, keys and values to their respective representations
-            
-        Returns a dict of (nested) Sections
-
-        @param configobj: a ConfigObj instance, basically a dict of (unparsed) sections
-        """
-        # note: configobj already converts comma-separated strings in lists
-        #
-        # list of supported keywords, all else will fail
-        #    versions: comma-separated list of version operators
-        #    toolchains: comma-separated list of toolchain version operators
-        SUPPORTED_KEYS = ('versions', 'toolchains')
-        if parent is None:
-            # no parent, so top sections
-            parsed_sections = {}
-        else:
-            # parent specified, so not a top section
-            parsed_sections = Section(parent=parent, depth=depth+1, main=configobj)
-
-        # start with full configobj initially, and then process subsections recursively
-        if toparse is None:
-            toparse = configobj
-
-        for key, value in toparse.items():
-            if isinstance(value, Section):
-                self.log.debug("Enter subsection key %s value %s" % (key, value))
-                # only 3 types of sectionkeys supported: VersionOperator, ToolchainVersionOperator, and DEFAULT
-                if key in [self.DEFAULT]:
-                    new_key = key
-                else:
-                    # try parsing key as toolchain version operator first
-                    # try parsing as version operator if it's not a toolchain version operator
-                    for marker_type in self.KNOWN_MARKER_TYPES:
-                        new_key = marker_type(key)
-                        if new_key:
-                            self.log.debug("'%s' was parsed as a %s section marker" % (key, marker_type.__name__))
-                            break
-                        else:
-                            self.log.debug("Not a %s section marker" % marker_type.__name__)
-                    if not new_key:
-                        self.log.error("Unsupported section marker '%s'" % key)
-
-                # parse value as a section, recursively
-                new_value = self.parse_sections(configobj, toparse=value, parent=value.parent, depth=value.depth)
-
-            else:
-                new_key = key
-
-                # don't allow any keys we don't know about (yet)
-                if not new_key in SUPPORTED_KEYS:
-                    self.log.error('Unsupported key %s with value %s in section' % (new_key, value))
-
-                # parse individual key-value assignments
-                if new_key in self.VERSION_OPERATOR_VALUE_TYPES:
-                    value_type = self.VERSION_OPERATOR_VALUE_TYPES[new_key]
-                    # list of supported toolchains/versions
-                    # first one is default
-                    if isinstance(value, basestring):
-                        # so the split should be unnecessary
-                        # (if it's not a list already, it's just one value)
-                        # TODO this is annoying. check if we can force this in configobj
-                        value = value.split(',')
-                    # remove possible surrounding whitespace (some people add space after comma)
-                    new_value = map(lambda x: value_type(x.strip()), value)
-                else:
-                    tup = (new_key, value, type(value))
-                    self.log.error('Bug: supported but unknown key %s with non-string value: %s, type %s' % tup)
-
-            self.log.debug('Converted key %s value %s in new key %s new value %s' % (key, value, new_key, new_value))
-            parsed_sections[new_key] = new_value
-
-        return parsed_sections
-
-    def validate_and_filter_by_toolchain(self, tcname, processed=None, filtered_sections=None, other_sections=None):
-        """
-        Build the ordered version operator and toolchain version operator, ignoring all other toolchains
-        @param tcname: toolchain name to keep
-        @param processed: a processed dict of sections to filter
-        @param path: list of keys to identify the path in the dict
-        """
-        top_call = False
-        if processed is None:
-            processed = self.sections
-            top_call = True
-        if filtered_sections is None:
-            filtered_sections = {}
-        if other_sections is None:
-            other_sections = {}
-
-        # walk over dictionary of parsed sections, and check for marker conflicts (using .add())
-        # add section markers relevant to specified toolchain to self.tcversops
-        for key, value in processed.items():
-            if isinstance(value, Section):
-                if isinstance(key, ToolchainVersionOperator):
-                    if not key.tc_name == tcname:
-                        self.log.debug("Found marker for other toolchain '%s'" % key.tc_name)
-                        # also perform sanity check for other toolchains, make add check for conflicts
-                        tc_overops = other_sections.setdefault(key.tc_name, OrderedVersionOperators())
-                        tc_overops.add(key)
-                        # nothing more to do here, just continue with other sections
-                        continue
-                    else:
-                        # add marker to self.tcversops (which triggers a conflict check)
-                        self.tcversops.add(key, value)
-                        filtered_sections[key] = value
-                elif isinstance(key, VersionOperator):
-                    # keep track of all version operators, and enforce conflict check
-                    self.versops.add(key, value)
-                    filtered_sections[key] = value
-                else:
-                    self.log.error("Unhandled section marker type '%s', not in %s?" % (type(key), self.KNOWN_MARKER_TYPES))
-
-                # recursively go deeper for (relevant) sections
-                self.validate_and_filter_by_toolchain(tcname, value, filtered_sections, other_sections)
-
-            elif key in self.VERSION_OPERATOR_VALUE_TYPES:
-                if key == 'toolchains':
-                    # remove any other toolchain from list
-                    filtered_sections[key] = [tcversop for tcversop in value if tcversop.tc_name == tcname]
-                else:
-                    # retain all other values
-                    filtered_sections[key] = value
-            else:
-                filtered_sections[key] = value
-
-        if top_call:
-            self.unfiltered_sections = self.sections
-            self.sections = filtered_sections
-
-    def parse(self, configobj):
-        """
-        First parse the configobj instance
-        Then build the structure to support the versionoperators and all other parts of the structure
-
-        @param configobj: ConfigObj instance
-        """
-        # keep reference to original (in case it's needed/wanted)
-        self.configobj = configobj
-
-        # process the configobj instance
-        self.sections = self.parse_sections(self.configobj)
-
-        # check for defaults section
-        default = self.sections.pop(self.DEFAULT, {})
-        DEFAULT_KEYWORDS = ('toolchains', 'versions')
-        # default should only have versions and toolchains
-        # no nesting
-        #  - add DEFAULT key,values to the root of self.sections
-        for key, value in default.items():
-            if not key in DEFAULT_KEYWORDS:
-                self.log.error('Unsupported key %s in %s section' % (key, self.DEFAULT))
-            self.sections[key] = value
-
-        if 'versions' in default:
-            # first of list is special: it is the default
-            default['default_version'] = default['versions'][0]
-        if 'toolchains' in default:
-            # first of list is special: it is the default
-            default['default_toolchain'] = default['toolchains'][0]
-
-        self.default = default
-        self.log.debug("parse: default %s, sections %s" % (self.default, self.sections))
