@@ -169,6 +169,9 @@ class ModulesTool(object):
         self.set_and_check_version()
         self.use_module_paths()
 
+        # this can/should be set to True during testing
+        self.testing = False
+
     def buildstats(self):
         """Return tuple with data to be included in buildstats"""
         return (self.__class__.__name__, self.cmd, self.version)
@@ -649,24 +652,30 @@ class Lmod(ModulesTool):
 
     def update(self):
         """Update after new modules were added."""
-        cmd = ['spider', '-o', 'moduleT', os.environ['MODULEPATH']]
+        spider_cmd = os.path.join(os.path.dirname(self.cmd), 'spider')
+        cmd = [spider_cmd, '-o', 'moduleT', os.environ['MODULEPATH']]
+        self.log.debug("Running command '%s'..." % ' '.join(cmd))
         proc = subprocess.Popen(cmd, stdout=PIPE, stderr=PIPE, env=os.environ)
         (stdout, stderr) = proc.communicate()
 
         if stderr:
             self.log.error("An error occured when running '%s': %s" % (' '.join(cmd), stderr))
 
-        try:
-            cache_filefn = os.path.join(os.path.expanduser('~'), '.lmod.d', '.cache', 'moduleT.lua')
-            self.log.debug("Updating Lmod spider cache %s with output from '%s'" % (cache_filefn, ' '.join(cmd)))
-            cache_dir = os.path.dirname(cache_filefn)
-            if not os.path.exists(cache_dir):
-                os.makedirs(cache_dir)
-            cache_file = open(cache_filefn, 'w')
-            cache_file.write(stdout)
-            cache_file.close()
-        except (IOError, OSError), err:
-            self.log.error("Failed to update Lmod spider cache %s: %s" % (cache_filefn, err))
+        if self.testing:
+            # don't actually update local cache when testing, just return the cache contents
+            return stdout
+        else:
+            try:
+                cache_filefn = os.path.join(os.path.expanduser('~'), '.lmod.d', '.cache', 'moduleT.lua')
+                self.log.debug("Updating Lmod spider cache %s with output from '%s'" % (cache_filefn, ' '.join(cmd)))
+                cache_dir = os.path.dirname(cache_filefn)
+                if not os.path.exists(cache_dir):
+                    os.makedirs(cache_dir)
+                cache_file = open(cache_filefn, 'w')
+                cache_file.write(stdout)
+                cache_file.close()
+            except (IOError, OSError), err:
+                self.log.error("Failed to update Lmod spider cache %s: %s" % (cache_filefn, err))
 
     def module_software_name(self, mod_name):
         """Get the software name for a given module name."""
