@@ -222,8 +222,8 @@ class GitRepository(FileRepository):
     Class for git repositories.
     """
     DESCRIPTION = ("A non-empty bare git repository (created with 'git init --bare' or 'git clone --bare'). "
-                   "The 1st argumentcontains the git repository location, which can be a directory or an URL. "
-                   "The second arhument  is a path inside the repository where to save the files.")
+                   "The 1st argument contains the git repository location, which can be a directory or an URL. "
+                   "The 2nd argument is a path inside the repository where to save the files.")
 
     USABLE = HAVE_GIT
 
@@ -330,8 +330,9 @@ class SvnRepository(FileRepository):
     Class for svn repositories
     """
 
-    DESCRIPTION = ("A SVN repository. The 1st argument contains the "
-                   "subversion repository location, this can be a directory or an URL.")
+    DESCRIPTION = ("An SVN repository. The 1st argument contains the "
+                   "subversion repository location, this can be a directory or an URL. "
+                   "The 2nd argument is a path inside the repository where to save the files.")
 
     USABLE = HAVE_PYSVN
 
@@ -456,7 +457,7 @@ def init_repository(repository, repository_path):
         try:
             if isinstance(repository_path, basestring):
                 return repo(repository_path)
-            elif isinstance(repository_path, (tuple, list)) and len(repository_path) == 2:
+            elif isinstance(repository_path, (tuple, list)) and len(repository_path) <= 2:
                 return repo(*repository_path)
             else:
                 _log.error('repository_path should be a string or list/tuple of maximum 2 elements (current: %s, type %s)' %
