@@ -1,5 +1,5 @@
 # #
-# Copyright 2012-2013 Ghent University
+# Copyright 2012-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -85,33 +85,6 @@ class FileToolsTest(TestCase):
             cmd = ft.extract_cmd(fn)
             self.assertEqual(expected_cmd, cmd)
 
-    def test_run_cmd(self):
-        """Basic test for run_cmd function."""
-        (out, ec) = ft.run_cmd("echo hello")
-        self.assertEqual(out, "hello\n")
-        # no reason echo hello could fail
-        self.assertEqual(ec, 0)
-
-    def test_run_cmd_bis(self):
-        """More 'complex' test for run_cmd function."""
-        # a more 'complex' command to run, make sure all required output is there
-        (out, ec) = ft.run_cmd("for j in `seq 1 3`; do for i in `seq 1 100`; do echo hello; done; sleep 1.4; done")
-        self.assertTrue(out.startswith('hello\nhello\n'))
-        self.assertEqual(len(out), len("hello\n"*300))
-        self.assertEqual(ec, 0)
-
-    def test_run_cmd_qa(self):
-        """Basic test for run_cmd_qa function."""
-        (out, ec) = ft.run_cmd_qa("echo question; read x; echo $x", {"question": "answer"})
-        self.assertEqual(out, "question\nanswer\n")
-        # no reason echo hello could fail
-        self.assertEqual(ec, 0)
-
-    def test_run_cmd_simple(self):
-        """Test return value for run_cmd in 'simple' mode."""
-        self.assertEqual(True, ft.run_cmd("echo hello", simple=True))
-        self.assertEqual(False, ft.run_cmd("exit 1", simple=True, log_all=False, log_ok=False))
-
     def test_convert_name(self):
         """Test convert_name function."""
         name = ft.convert_name("test+test-test")
@@ -119,38 +92,11 @@ class FileToolsTest(TestCase):
         name = ft.convert_name("test+test-test", True)
         self.assertEqual(name, "TESTPLUSTESTMINTEST")
 
-    def test_parse_log_error(self):
-        """Test basic parse_log_for_error functionality."""
-        errors = ft.parse_log_for_error("error failed", True)
-        self.assertEqual(len(errors), 1)
-
-        # I expect tests to be run from the base easybuild directory
+    def test_cwd(self):
+        """tests should be run from the base easybuild directory"""
+        # used to be part of test_parse_log_error
         self.assertEqual(os.getcwd(), ft.find_base_dir())
 
-    def test_run_cmd_suse(self):
-        """Test run_cmd on SuSE systems, which have $PROFILEREAD set."""
-        # avoid warning messages
-        ft_log_level = ft._log.getEffectiveLevel()
-        ft._log.setLevel('ERROR')
-
-        # run_cmd should also work if $PROFILEREAD is set (very relevant for SuSE systems)
-        profileread = os.environ.get('PROFILEREAD', None)
-        os.environ['PROFILEREAD'] = 'profilereadxxx'
-        try:
-            (out, ec) = ft.run_cmd("echo hello")
-        except Exception, err:
-            out, ec = "ERROR: %s" % err, 1
-
-        # make sure it's restored again before we can fail the test
-        if profileread is not None:
-            os.environ['PROFILEREAD'] = profileread
-        else:
-            del os.environ['PROFILEREAD']
-
-        self.assertEqual(out, "hello\n")
-        self.assertEqual(ec, 0)
-        ft._log.setLevel(ft_log_level)
-
     def test_encode_class_name(self):
         """Test encoding of class names."""
         for (class_name, encoded_class_name) in self.class_names:
@@ -179,8 +125,8 @@ class FileToolsTest(TestCase):
         ft.write_file(fp, perltxt)
         ft.patch_perl_script_autoflush(fp)
         txt = ft.read_file(fp)
-        self.assertTrue(len(txt.split('\n')) == len(perl_lines)+4)
-        self.assertTrue(txt.startswith(perl_lines[0]+"\n\nuse IO::Handle qw();\nSTDOUT->autoflush(1);"))
+        self.assertTrue(len(txt.split('\n')) == len(perl_lines) + 4)
+        self.assertTrue(txt.startswith(perl_lines[0] + "\n\nuse IO::Handle qw();\nSTDOUT->autoflush(1);"))
         for line in perl_lines[1:]:
             self.assertTrue(line in txt)
         os.remove(fp)
