@@ -1,5 +1,5 @@
 # #
-# Copyright 2013 Ghent University
+# Copyright 2013-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -37,6 +37,7 @@ import tempfile
 from unittest import TestCase, TestLoader
 from unittest import main as unittestmain
 
+import easybuild.tools.build_log
 import easybuild.tools.options as eboptions
 from easybuild.main import main
 from easybuild.framework.easyconfig import BUILD, CUSTOM, DEPENDENCIES, EXTENSIONS, FILEMANAGEMENT, LICENSE
@@ -46,6 +47,7 @@ from easybuild.tools.environment import modify_env
 from easybuild.tools.filetools import read_file, write_file
 from easybuild.tools.modules import modules_tool
 from easybuild.tools.options import EasyBuildOptions
+from easybuild.tools.version import VERSION
 from vsc import fancylogger
 
 class CommandLineOptionsTest(TestCase):
@@ -62,8 +64,22 @@ class CommandLineOptionsTest(TestCase):
         # create log file
         fd, self.logfile = tempfile.mkstemp(suffix='.log', prefix='eb-options-test-')
         os.close(fd)
-        # keep track of original environment to restore
+        # keep track of original environment/Python search path to restore
         self.orig_environ = copy.deepcopy(os.environ)
+        self.orig_sys_path = sys.path[:]
+
+        # (re)import and reload easybuild modules
+        import easybuild
+        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'sandbox')))
+        easybuild = reload(easybuild)
+        import easybuild.easyblocks
+        reload(easybuild.easyblocks)
+        import easybuild.easyblocks.generic
+        reload(easybuild.easyblocks.generic)
+        reload(easybuild.tools.module_naming_scheme)  # required to run options unit tests stand-alone
+
+        # set MODULEPATH to included test modules
+        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
 
     def tearDown(self):
         """Post-test cleanup."""
@@ -72,6 +88,9 @@ class CommandLineOptionsTest(TestCase):
         modify_env(os.environ, self.orig_environ)
         tempfile.tempdir = None
 
+        # restore original Python search path
+        sys.path = self.orig_sys_path
+
     def test_help_short(self, txt=None):
         """Test short help message."""
 
@@ -199,9 +218,6 @@ class CommandLineOptionsTest(TestCase):
     def test_force(self):
         """Test forcing installation even if the module is already available."""
 
-        # set MODULEPATH to included modules
-        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
-
         # use GCC-4.6.3.eb easyconfig file that comes with the tests
         eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'GCC-4.6.3.eb')
 
@@ -251,9 +267,6 @@ class CommandLineOptionsTest(TestCase):
         installpath = tempfile.mkdtemp()
         sourcepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sandbox', 'sources')
 
-        # set MODULEPATH to included modules
-        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
-
         # use toy-0.0.eb easyconfig file that comes with the tests
         eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'toy-0.0.eb')
 
@@ -285,6 +298,7 @@ class CommandLineOptionsTest(TestCase):
         modules_tool().purge()
         # reinitialize modules tool with original $MODULEPATH, to avoid problems with future tests
         modify_env(os.environ, self.orig_environ)
+        os.environ['MODULEPATH'] = ''
         modules_tool()
         tempfile.tempdir = None
 
@@ -326,9 +340,6 @@ class CommandLineOptionsTest(TestCase):
     def test_job(self):
         """Test submitting build as a job."""
 
-        # set MODULEPATH to included modules
-        os.environ['MODULEPATH'] = os.path.join(os.path.dirname(__file__), 'modules')
-
         # use gzip-1.4.eb easyconfig file that comes with the tests
         eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'gzip-1.4.eb')
 
@@ -459,25 +470,10 @@ class CommandLineOptionsTest(TestCase):
             if os.path.exists(dummylogfn):
                 os.remove(dummylogfn)
 
-        # also check whether available custom easyconfig parameters are listed
-        orig_sys_path = sys.path[:]
-
-        import easybuild
-        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'sandbox')))
-        easybuild = reload(easybuild)
-        import easybuild.easyblocks
-        reload(easybuild.easyblocks)
-        import easybuild.easyblocks.generic
-        reload(easybuild.easyblocks.generic)
-        reload(easybuild.tools.module_naming_scheme)  # required to run options unit tests stand-alone
-
         run_test(custom='EB_foo', extra_params=['foo_extra1', 'foo_extra2'])
         run_test(custom='bar', extra_params=['bar_extra1', 'bar_extra2'])
         run_test(custom='EB_foofoo', extra_params=['foofoo_extra1', 'foofoo_extra2'])
 
-        # restore original Python search path
-        sys.path = orig_sys_path
-
     # double underscore to make sure it runs first, which is required to detect certain types of bugs,
     # e.g. running with non-initialized EasyBuild config (truly mimicing 'eb --list-toolchains')
     def test__list_toolchains(self):
@@ -552,7 +548,6 @@ class CommandLineOptionsTest(TestCase):
         os.close(fd)
 
         # adjust PYTHONPATH such that test easyblocks are found
-        orig_sys_path = sys.path[:]
 
         import easybuild
         sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'sandbox')))
@@ -613,9 +608,6 @@ class CommandLineOptionsTest(TestCase):
         if os.path.exists(dummylogfn):
             os.remove(dummylogfn)
 
-        # restore original Python search path
-        sys.path = orig_sys_path
-
     def test_search(self):
         """Test searching for easyconfigs."""
 
@@ -742,6 +734,93 @@ class CommandLineOptionsTest(TestCase):
         msg = "Error message when eb can't find software with specified name (myerr: %s, outtxt: %s)" % (myerr, outtxt)
         self.assertTrue(re.search(error_msg1, outtxt) or re.search(error_msg2, outtxt), msg)
 
+    def test_footer(self):
+        """Test specifying a module footer."""
+        # use temporary paths for build/install paths, make sure sources can be found
+        buildpath = tempfile.mkdtemp()
+        installpath = tempfile.mkdtemp()
+        tmpdir = tempfile.mkdtemp()
+        sourcepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sandbox', 'sources')
+
+        # create file containing modules footer
+        module_footer_txt = '\n'.join([
+            "# test footer",
+            "setenv SITE_SPECIFIC_ENV_VAR foobar",
+        ])
+        fd, modules_footer = tempfile.mkstemp(prefix='modules-footer-')
+        os.close(fd)
+        f = open(modules_footer, 'w')
+        f.write(module_footer_txt)
+        f.close()
+
+        # use toy-0.0.eb easyconfig file that comes with the tests
+        eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'toy-0.0.eb')
+
+        # check log message with --skip for existing module
+        args = [
+            eb_file,
+            '--sourcepath=%s' % sourcepath,
+            '--buildpath=%s' % buildpath,
+            '--installpath=%s' % installpath,
+            '--debug',
+            '--force',
+            '--modules-footer=%s' % modules_footer,
+        ]
+
+        try:
+            main((args, self.logfile, True))
+        except (SystemExit, Exception), err:
+            pass
+
+        toy_module = os.path.join(installpath, 'modules', 'all', 'toy', '0.0')
+        toy_module_txt = read_file(toy_module)
+        footer_regex = re.compile(r'%s$' % module_footer_txt, re.M)
+        msg = "modules footer '%s' is present in '%s'" % (module_footer_txt, toy_module_txt)
+        self.assertTrue(footer_regex.search(toy_module_txt), msg)
+
+        # cleanup
+        shutil.rmtree(buildpath)
+        shutil.rmtree(installpath)
+        shutil.rmtree(tmpdir)
+        os.remove(modules_footer)
+
+    def test_recursive_module_unload(self):
+        """Test generating recursively unloading modules."""
+
+        # use temporary paths for build/install paths, make sure sources can be found
+        buildpath = tempfile.mkdtemp()
+        installpath = tempfile.mkdtemp()
+        tmpdir = tempfile.mkdtemp()
+        sourcepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sandbox', 'sources')
+
+        # use toy-0.0.eb easyconfig file that comes with the tests
+        eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'toy-0.0-deps.eb')
+
+        # check log message with --skip for existing module
+        args = [
+            eb_file,
+            '--sourcepath=%s' % sourcepath,
+            '--buildpath=%s' % buildpath,
+            '--installpath=%s' % installpath,
+            '--debug',
+            '--force',
+            '--recursive-module-unload',
+        ]
+
+        try:
+            main((args, self.logfile, True))
+        except (SystemExit, Exception), err:
+            pass
+
+        toy_module = os.path.join(installpath, 'modules', 'all', 'toy', '0.0-deps')
+        toy_module_txt = read_file(toy_module)
+        is_loaded_regex = re.compile(r"if { !\[is-loaded gompi/1.3.12\] }", re.M)
+        self.assertFalse(is_loaded_regex.search(toy_module_txt), "Recursive unloading is used: %s" % toy_module_txt)
+
+        # cleanup
+        shutil.rmtree(buildpath)
+        shutil.rmtree(installpath)
+        shutil.rmtree(tmpdir)
 
     def test_tmpdir(self):
         """Test setting temporary directory to use by EasyBuild."""
@@ -752,9 +831,6 @@ class CommandLineOptionsTest(TestCase):
         tmpdir = tempfile.mkdtemp()
         sourcepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sandbox', 'sources')
 
-        # set MODULEPATH to included modules
-        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
-
         # use toy-0.0.eb easyconfig file that comes with the tests
         eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'toy-0.0.eb')
 
@@ -801,7 +877,7 @@ class CommandLineOptionsTest(TestCase):
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
             'toolchain = {"name":"dummy", "version": "dummy"}',
-            'osdependencies = ["nosuchosdependency"]',
+            'osdependencies = ["nosuchosdependency", ("nosuchdep_option1", "nosuchdep_option2")]',
         ])
         fd, eb_file = tempfile.mkstemp(prefix='easyconfig_test_file_', suffix='.eb')
         os.close(fd)
@@ -819,7 +895,9 @@ class CommandLineOptionsTest(TestCase):
         outtxt = read_file(self.logfile)
         regex = re.compile("Checking OS dependencies")
         self.assertTrue(regex.search(outtxt), "OS dependencies are checked, outtxt: %s" % outtxt)
-        regex = re.compile("One or more OS dependencies were not found: \['nosuchosdependency'\]", re.M)
+        msg = "One or more OS dependencies were not found: "
+        msg += "\[\('nosuchosdependency',\), \('nosuchdep_option1', 'nosuchdep_option2'\)\]"
+        regex = re.compile(r'%s' % msg, re.M)
         self.assertTrue(regex.search(outtxt), "OS dependencies are honored, outtxt: %s" % outtxt)
 
         # check whether OS dependencies are effectively ignored
@@ -851,6 +929,72 @@ class CommandLineOptionsTest(TestCase):
         regex = re.compile("stop provided 'notavalidstop' is not valid", re.M)
         self.assertTrue(regex.search(outtxt), "Validations are performed with --ignore-osdeps, outtxt: %s" % outtxt)
 
+    def test_experimental(self):
+        """Test the experimental option"""
+        orig_value = easybuild.tools.build_log.EXPERIMENTAL
+        # make sure it's off by default
+        self.assertFalse(orig_value)
+
+        log = fancylogger.getLogger()
+
+        # force it to False
+        topt = EasyBuildOptions(
+            go_args=['--disable-experimental'],
+        )
+        try:
+            log.experimental('x')
+            # sanity check, should never be reached if it works.
+            self.assertTrue(False, "Experimental logging should be disabled by setting the --disable-experimental option")
+        except easybuild.tools.build_log.EasyBuildError, err:
+            # check error message
+            self.assertTrue('Experimental functionality.' in str(err))
+
+        # toggle experimental
+        topt = EasyBuildOptions(
+            go_args=['--experimental'],
+        )
+        try:
+            log.experimental('x')
+        except easybuild.tools.build_log.EasyBuildError, err:
+            self.assertTrue(False, 'Experimental logging should be allowed by the --experimental option.')
+
+        # set it back
+        easybuild.tools.build_log.EXPERIMENTAL = orig_value
+
+    def test_deprecated(self):
+        """Test the deprecated option"""
+        orig_value = easybuild.tools.build_log.CURRENT_VERSION
+
+        # make sure it's off by default
+        self.assertEqual(orig_value, VERSION)
+
+        log = fancylogger.getLogger()
+
+        # force it to current version
+        topt = EasyBuildOptions(
+            go_args=['--deprecated=%s' % orig_value],
+        )
+
+        try:
+            log.deprecated('x', str(orig_value))
+        except easybuild.tools.build_log.EasyBuildError, err:
+            self.assertTrue(False, 'Deprecated logging should work')
+
+        # force higher version by prefixing it with 1
+        topt = EasyBuildOptions(
+            go_args=['--deprecated=1%s' % orig_value],
+        )
+        try:
+            log.deprecated('x', str(orig_value))
+            # not supposed to get here
+            self.assertTrue(False, 'Deprecated logging should throw EasyBuildError')
+        except easybuild.tools.build_log.EasyBuildError, err2:
+            self.assertTrue('DEPRECATED' in str(err2))
+
+        # set it back
+        easybuild.tools.build_log.CURRENT_VERSION = orig_value
+
+
 def suite():
     """ returns all the testcases in this module """
     return TestLoader().loadTestsFromTestCase(CommandLineOptionsTest)
